import { S as StoredObject, R as ReducedFunction, a as Replace, b as ReplaceRecursively, c as StorableObjectValue, Q as QueryHandlerOptionsFactory, P as PromiseTuple } from './index-YWnXwe92.js';
import { Records } from './schema/index.js';
import 'node:async_hooks';

declare namespace RONIN {
    export interface RoninRecord<TId extends string = string> {
        id: TId;
        ronin: RoninMetadata;
    }
    export interface RoninMetadata {
        createdAt: Date;
        createdBy: string | Record<string, any>;
        deletedAt: Date | null;
        deletedBy: string | Record<string, any> | null;
        locked: boolean;
        status: 'draft' | 'published' | 'archived';
        updatedAt: Date;
        updatedBy: string | Record<string, any>;
    }
    export interface Blob extends StoredObject {
    }
    interface StringFilterFunction<T, R, O> extends ReducedFunction {
        (value: T, options?: O): Promise<R>;
        /**
         * Returns records where the field is not equal to the provided value.
         */
        notBeing: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field starts with the provided value.
         */
        startingWith: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field ends with the provided value.
         */
        endingWith: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field contains the provided value.
         */
        containing: (value: T, options?: O) => Promise<R>;
    }
    interface NumberFilterFunction<T, R, O> extends ReducedFunction {
        (value: T, options?: O): Promise<R>;
        /**
         * Returns records where the field is not equal to the provided value.
         */
        notBeing: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field is greater than the provided value.
         */
        greaterThan: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field is less than the provided value.
         */
        lessThan: (value: T, options?: O) => Promise<R>;
    }
    interface DateFilterFunction<T, R, O> extends ReducedFunction {
        (value: T, options?: O): Promise<R>;
        /**
         * Returns records where the field is not equal to the provided value.
         */
        notBeing: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field is greater than the provided value.
         */
        greaterThan: (value: T, options?: O) => Promise<R>;
        /**
         * Returns records where the field is less than the provided value.
         */
        lessThan: (value: T, options?: O) => Promise<R>;
    }
    interface BooleanFilterFunction<T, R, O> extends ReducedFunction {
        (value: T, options?: O): Promise<R>;
    }
    interface RecordFilterFunction<R, O> extends ReducedFunction {
        (value: string, options?: O): Promise<R>;
    }
    type RecordFilterObject<T, R, O> = {
        [K in keyof T]: FilterFunction<T[K], R, O>;
    };
    type RecordFilter<T, R, O> = RecordFilterFunction<R, O> & RecordFilterObject<T, R, O>;
    type FilterFunction<T, R, O> = T extends string ? StringFilterFunction<T, R, O> : T extends number ? NumberFilterFunction<T, R, O> : T extends boolean ? BooleanFilterFunction<T, R, O> : T extends Date ? DateFilterFunction<T, R, O> : T extends RONIN.RoninRecord<string> ? RecordFilter<T, R, O> : T extends Record<string, any> ? RecordFilterObject<T, R, O> : never;
    type FilterObject<T> = T extends string ? {
        /**
         * `being` instruction can't be used in combination with other
         * search instructions.
         */
        being: string;
        notBeing: never;
        startingWith: never;
        endingWith: never;
        containing: never;
    } | {
        /**
         * `notBeing` instruction can't be used in combination with other
         * search instructions.
         */
        notBeing: string;
        being: never;
        startingWith: never;
        endingWith: never;
        containing: never;
    } | {
        being: never;
        notBeing: never;
        /**
         * Matches records where the field starts with the provided value.
         */
        startingWith?: string;
        /**
         * Matches records where the field ends with the provided value.
         */
        endingWith?: string;
        /**
         * Matches records where the field contains the provided value.
         */
        containing?: string;
    } : T extends number ? {
        /**
         * `being` instruction can't be used in combination with other
         * search instructions.
         */
        being: number;
        notBeing: never;
        greaterThan: never;
        lessThan: never;
    } | {
        /**
         * `notBeing` instruction can't be used in combination with other
         * search instructions.
         */
        notBeing: number;
        being: never;
        greaterThan: never;
        lessThan: never;
    } | {
        being: never;
        notBeing: never;
        /**
         * Matches records where the field is greater than the provided value.
         */
        greaterThan?: number;
        /**
         * Matches records where the field is less than the provided value.
         */
        lessThan?: number;
    } : T extends Date ? {
        /**
         * `being` instruction can't be used in combination with other
         * search instructions.
         */
        being: Date;
        notBeing: never;
        greaterThan: never;
        lessThan: never;
    } | {
        /**
         * `notBeing` instruction can't be used in combination with other
         * search instructions.
         */
        notBeing: Date;
        being: never;
        greaterThan: never;
        lessThan: never;
    } | {
        being: never;
        notBeing: never;
        /**
         * Matches records where the field is greater than the provided value.
         */
        greaterThan?: Date;
        /**
         * Matches records where the field is less than the provided value.
         */
        lessThan?: Date;
    } : T extends boolean ? {
        /**
         * Matches records where the field is equal to the provided value.
         */
        being: boolean;
    } : T extends RONIN.RoninRecord<string> ? string | {
        [K in keyof T]: T[K] | Partial<FilterObject<T[K]>>;
    } : T extends Record<string, any> ? {
        [K in keyof T]: T[K] | Partial<FilterObject<T[K]>>;
    } : never;
    export type WithObject<TSchema> = {
        [K in keyof TSchema]: TSchema[K] | Partial<FilterObject<TSchema[K]>>;
    };
    export type WithFilterFunctions<TSchema, R, O = undefined> = {
        [K in keyof TSchema]: FilterFunction<TSchema[K], R, O>;
    };
    type WithFunction<TSchema, TReturn, TOptions> = Omit<ReducedFunction, keyof TSchema> & {
        (filter: Partial<WithObject<TSchema>>, options?: TOptions): Promise<TReturn>;
    };
    type With<TSchema, R, O> = WithFunction<TSchema, R, O> & WithFilterFunctions<TSchema, R>;
    type AllFields<TSchema> = `ronin.${keyof RoninMetadata}` | Exclude<keyof TSchema, 'ronin'>;
    type OrderedByObject<TSchema> = {
        /**
         * Order the resulting records in descending order using a specific field.
         */
        descending?: Array<AllFields<TSchema>>;
        /**
         * Order the resulting records in ascending order using a specific field.
         */
        ascending?: Array<AllFields<TSchema>>;
    };
    interface OrderedByFunction<TSchema, R, O> extends ReducedFunction {
        (order: OrderedByObject<TSchema>, options?: O): Promise<R>;
        /**
         * Returns records in descending order using a specific field.
         */
        descending: (order: Array<AllFields<TSchema>>, options?: O) => Promise<R>;
        /**
         * Returns records in ascending order using a specific field.
         */
        ascending: (order: Array<AllFields<TSchema>>, options?: O) => Promise<R>;
    }
    type RelatedFieldKeys<T> = {
        [K in keyof T]: T[K] extends RONIN.RoninRecord<string> ? (K extends string ? K : never) : never;
    }[keyof T];
    export type Including<T> = RelatedFieldKeys<T>[] | 'all';
    export type ReturnBasedOnIncluding<T, Keys extends string[] | 'all'> = Keys extends 'all' ? T : {
        [K in keyof T]: K extends 'ronin' ? T[K] : K extends Keys[number] ? T[K] : T[K] extends RONIN.RoninRecord ? string : T[K];
    };
    export interface IGetterSingular<TSchema, TVariant extends string = string, TOptions = undefined, TModifiedReturn = Replace<TSchema, RONIN.RoninRecord, string>> extends ReducedFunction {
        <TIncluding extends Including<TSchema> = []>(filter?: {
            with?: Partial<WithObject<TSchema>>;
            in?: TVariant;
            including?: TIncluding;
        }, options?: TOptions): Promise<ReturnBasedOnIncluding<TSchema, TIncluding> | null>;
        with: With<TSchema, TModifiedReturn | null, TOptions>;
    }
    export interface IGetterPlural<TSchema, TVariant extends string = string, TOptions = undefined, TModifiedReturn = Records<Replace<TSchema, RONIN.RoninRecord, string>>> extends ReducedFunction {
        <TIncluding extends Including<TSchema> = []>(filter?: {
            with?: Partial<WithObject<TSchema>>;
            orderedBy?: OrderedByObject<TSchema>;
            limitedTo?: number;
            in?: TVariant;
            including?: TIncluding;
            after?: string;
            before?: string;
        }, options?: TOptions): Promise<Records<ReturnBasedOnIncluding<TSchema, TIncluding>>>;
        with: With<TSchema, TModifiedReturn, TOptions>;
        orderedBy: OrderedByFunction<TSchema, TModifiedReturn, TOptions>;
        limitedTo: (limit: number, options?: TOptions) => Promise<TModifiedReturn>;
        in: (variant: TVariant, options?: TOptions) => Promise<TModifiedReturn>;
        including: <TIncluding extends Including<TSchema> = []>(values: TIncluding, options?: TOptions) => Promise<Records<ReturnBasedOnIncluding<TSchema, TIncluding>>>;
        after: (cursor: string, options?: TOptions) => Promise<TModifiedReturn>;
        before: (cursor: string, options?: TOptions) => Promise<TModifiedReturn>;
    }
    export interface ISetter<TSchema, TVariant extends string = string, TOptions = undefined, TModifiedReturn = Replace<TSchema, RONIN.RoninRecord, string>> extends ReducedFunction {
        (filter: {
            with: Partial<WithObject<TSchema>>;
            to: Partial<ReplaceRecursively<TSchema, RONIN.Blob, StorableObjectValue>>;
            in?: TVariant;
        }, options?: TOptions): Promise<TModifiedReturn>;
    }
    export interface ICreator<TSchema, TVariant extends string = string, TOptions = undefined, TModifiedReturn = Replace<TSchema, RONIN.RoninRecord, string>> extends ReducedFunction {
        (filter?: {
            with: Partial<ReplaceRecursively<TSchema, RONIN.Blob, StorableObjectValue>>;
            in?: TVariant;
        }, options?: TOptions): Promise<TModifiedReturn>;
        with: (values: Partial<ReplaceRecursively<TSchema, RONIN.Blob, StorableObjectValue>>, options?: TOptions) => Promise<TModifiedReturn>;
    }
    export interface ICounter<TSchema, TVariant extends string = string, TOptions = undefined> extends ReducedFunction {
        (filter?: {
            with?: Partial<WithObject<TSchema>>;
            in?: TVariant;
        }): Promise<number>;
        with: With<TSchema, number, TOptions>;
        in: (variant: TVariant, options?: TOptions) => Promise<number>;
    }
    export interface IDropper<TSchema, TVariant extends string = string, TOptions = undefined, TModifiedReturn = Replace<TSchema, RONIN.RoninRecord, string>> extends ReducedFunction {
        (filter?: {
            with?: Partial<WithObject<TSchema>>;
            in?: TVariant;
        }, options?: TOptions): Promise<TModifiedReturn>;
        with: With<TSchema, TSchema, TOptions>;
        in: (variant: TVariant, options?: TOptions) => Promise<TModifiedReturn>;
    }
    export interface Creator<TOptions = undefined> extends Record<string, ICreator<Record<string, unknown>, string, TOptions>> {
    }
    export interface Getter<TOptions = undefined> extends Record<string, IGetterPlural<Record<string, unknown>, string, TOptions>> {
    }
    export interface Setter<TOptions = undefined> extends Record<string, ISetter<Record<string, unknown>, string, TOptions>> {
    }
    export interface Dropper<TOptions = undefined> extends Record<string, IDropper<Record<string, unknown>, string, TOptions>> {
    }
    export interface Counter<TOptions = undefined> extends Record<string, ICounter<Record<string, unknown>, string, TOptions>> {
    }
    export {  };
}

/**
 * Creates a syntax factory for generating and executing queries.
 *
 * @param options - An optional object of options for the query execution.
 *
 * Alternatively, a function that returns the object may be provided instead,
 * which is useful for cases in which the config must be generated dynamically
 * whenever a query is executed.
 *
 * @returns An object with methods for generating and executing different types
 * of queries.
 *
 * ### Usage
 * ```typescript
 * const { get, set, create, count, drop } = createSyntaxFactory({
 *   token: '...'
 * });
 *
 * await get.accounts();
 *
 * await set.account({
 *   with: {
 *     email: 'mike@gmail.com',
 *   },
 *   to: {
 *     status: 'active',
 *   },
 * });
 *
 *
 * await create.account({ with: { email: 'mike@gmail.com' } });
 *
 * await count.accounts();
 *
 * await drop.accounts.with.emailVerified.notBeing(true);
 *
 * // Execute a batch of operations
 * const batchResult = await batch(() => [
 *   get.accounts(),
 *   get.account.with.email('mike@gmail.com')
 * ]);
 * ```
 */
declare const createSyntaxFactory: (options: QueryHandlerOptionsFactory) => {
    create: RONIN.Creator;
    get: RONIN.Getter;
    set: RONIN.Setter;
    drop: RONIN.Dropper;
    count: RONIN.Counter;
    batch: <T extends [Promise<any>, ...Promise<any>[]]>(operations: () => T, batchQueryOptions?: Record<string, unknown>) => Promise<PromiseTuple<T>>;
};

declare const create: RONIN.Creator<undefined>;
declare const get: RONIN.Getter<undefined>;
declare const set: RONIN.Setter<undefined>;
declare const drop: RONIN.Dropper<undefined>;
declare const count: RONIN.Counter<undefined>;
declare const batch: <T extends [Promise<any>, ...Promise<any>[]]>(operations: () => T, batchQueryOptions?: Record<string, unknown>) => Promise<PromiseTuple<T>>;

export { RONIN, batch, count, create, createSyntaxFactory as default, drop, get, set };
